#from Interface import ZContact
from AccessControl import ClassSecurityInfo, getSecurityManager
from Products.ZLDAPMethods.LM import LDAPFilter, LDAPConnectionIDs
from Acquisition import Implicit
from Globals import Persistent
from Globals import DTMLFile, HTMLFile, MessageDialog
from AccessControl.Role import RoleManager
from OFS.SimpleItem import Item
import re

# A list of the attributes that updateEntry() and addEntry() use
mod_attrs = ('cn','sn','mail','telephonenumber','o','postaladdress','facsimiletelephonenumber')
# A list of attributes that the search uses
search_attrs = ('o','cn','mail')
# Apart from postaladdress and cn, what attributes to show in search results
result_attrs = (['Organisaatio','o'],
                ['Puhelinnumero','telephonenumber'],
                ['Faksinumero','facsimiletelephonenumber'])

#
# we need to filter spaces from dn's ...
#
dn_re = re.compile(r',\s+')

objAddForm = HTMLFile('dtml/objAddForm', globals())

def objAddFunction(dispatcher, id, connection_id, basedn, parentdn, objectclass, REQUEST=None):
    """ Create a new ZContact object and add it """
    p=ZContactProduct(id, connection_id, basedn, parentdn, objectclass)
    dispatcher.Destination()._setObject(id, p)

    if REQUEST is not None:
        u = REQUEST['URL1']
        u += '/manage_workspace'
        REQUEST.RESPONSE.redirect(u)
    return None

class ZContactProduct(Implicit, Persistent, RoleManager, Item):

    meta_type = 'ZContact'
    security = ClassSecurityInfo()

    manage_options = (
        {'label' : 'Properties', 'action':'manage_main'},
        {'label' : 'Security', 'action':'manage_access'},
        {'label' : 'Search contacts', 'action':'search'},
        {'label' : 'Add contacts', 'action':'add'}
    )
    __ac_permissions__= (
        ('View management screens', ('manage_tabs','manage_main',) ),
        ('Change permissions', ('manage_access',) ),
        ('View'), ('displaySearch','getEntryByDN', 'UTF8ToLatin', 'search'),
        ('Modify contact information', ('updateEntry', 'deleteEntry', 'addEntry', 'add')  )
        )

    def __init__(self, id, connection_id, basedn, parentdn, objectclass):
        self.id = id
        self._set_properties(connection_id, basedn, parentdn, objectclass)
        self._setupLDAPFilters()

    def _set_properties(self, connection_id, basedn, parentdn, objectclass):
        """ Sets the class attributes connection_id and basedn """
        if connection_id:
            self.connection_id = connection_id
        else:
            raise AttributeError, 'empty connection_id?'
        if basedn:
            self.basedn = basedn
        else:
            raise AttributeError, 'empty basedn?'
        if parentdn:
            self.parentdn = parentdn
        else:
            raise AttributeError, 'empty parentdn?'
        if objectclass:
            self.objectclass = objectclass
        else:
            raise AttributeError, 'empty objectclass?'

    ##
    # DTML methods

    # These will be published
    search = DTMLFile('dtml/search', globals())
    add = DTMLFile('dtml/add', globals())
    edit = DTMLFile('dtml/edit', globals())
    manage_main = DTMLFile('dtml/manage_propertiesForm', globals())
    # private
    displaySearch = DTMLFile('dtml/displaySearch', globals())
    menu = DTMLFile('dtml/menu', globals())


    ##
    # Published class methods

    def index_html(self, REQUEST=None):
        """
        If accessing the object as such, let's redirect to the searching form.
        """
        if REQUEST is not None:
            u = REQUEST['URL1']
            u += '/search'
            REQUEST.RESPONSE.redirect(u)
        return None

    def manage_properties(self, connection_id, basedn, parentdn, objectclass, REQUEST = None):
        """
        Does the necessary modifications when the basedn or connection ID
        is changed
        """
        self._set_properties(connection_id, basedn, parentdn, objectclass)
        self._setupLDAPFilters()
        if REQUEST is not None:
            return MessageDialog(
                title='Properties updated',
                message='The properties of %s were updated.' % self.id,
                action='manage_main')


    def updateEntry(self, dn, cn, mail='', o='', postaladdress='', telephonenumber='',
                    facsimiletelephonenumber='', REQUEST=None):
        """
        Update the entry in the database according to arguments passed via
        REQUEST

        Returns a tuple of the message telling whether the update was
        succesful or not, and the new dn.
        """
        (message, entryAttrs) = self._checkAttributes( {'cn':cn, 'mail':mail, 'o':o,
                                                        'telephonenumber':telephonenumber,
                                                        'postaladdress':postaladdress,
                                                        'facsimiletelephonenumber':facsimiletelephonenumber} )
        if not entryAttrs:
            if REQUEST is not None:
                REQUEST.form['msg'] = message
                return self.edit(self, REQUEST)
            return (message, dn)

        print str(entryAttrs)
        
        entry = self.getEntryByDN(dn)
        # We have to change the RDN if the cn-attribute changes
        newdn = entry.rename('cn='+entryAttrs['cn'])
        entry.setAll(entryAttrs)

        if REQUEST is not None:
            return MessageDialog(message="Modified!",
                                 title='ZContacts',
                                 action="%s/search" % REQUEST['URL1'])
        return newdn

    def addEntry(self, cn='', mail='', o='', telephonenumber='', facsimiletelephonenumber='',
                 postaladdress='', REQUEST=None):
        """
        Add an entry in the database according to arguments passed via
        REQUEST

        Returns a string indicating failure or success
        """
        (message, entryAttrs) = self._checkAttributes({'cn':cn, 'mail':mail, 'o':o,            
                                                       'telephonenumber':telephonenumber,
                                                       'postaladdress':postaladdress,
                                                       'facsimiletelephonenumber':facsimiletelephonenumber})
        if not entryAttrs:
            return message
        if hasattr(self, 'objectclass'):
            entryAttrs['objectclass'] = self.objectclass

        print str(entryAttrs)
        # The new entry will be added directly under the root entry
        parentEntry = self.getEntryByDN(self.parentdn)
        parentEntry.addSubentry('cn='+entryAttrs['cn'], entryAttrs)
        if REQUEST is not None:
            return MessageDialog(message="Added!",
                                 title='ZContacts',
                                 action="%s/search" % REQUEST['URL1'])

    def getEntryByDN(self, dn, targetEntry='', REQUEST=None):
        """
        Return the entry object that has the DN of REQUEST['dn']

        This had to be implemented in an eccentric way, that requires
        every entry to have an attribute identical to it's RDN, because the
        base used by LDAP filter objects without constructing them again.
        """
        if targetEntry:
            return targetEntry

        rdn = self._getRDNFromDN(dn)
        entries = self.getEntryLDAPFilter(clause = rdn)
        
        for entry in entries:
            #
            # remove whitespace from within dn elements to assure commonality
            # between disparate dn formatters ...
            #
            entry_dn = dn_re.sub(',', entry.dn)
            request_dn = dn_re.sub(',', dn)
            if entry_dn == request_dn:
                return entry

        raise Exception, 'Entry not found? ' + dn + ' : ' + rdn

    def deleteEntry(self, dn, REQUEST=None):
        """
        """
        targetEntry = self.getEntryByDN(dn)
        targetEntry.deleteSubentry(targetEntry)

        if REQUEST is not None:
            return MessageDialog(message = 'Removed',
                                 title = 'ZContact',
                                 action= '%s/manage_main' % REQUEST.URL1)

        
    security.declarePrivate('decodePostaladdress')
    def decodePostaladdress(self, postaladdress):
        """
        Changes $s to newlines, used to decode the postaladdress before
        putting showing it in a form.
        """
        string = str(postaladdress)
        return string.replace('$', '\n')

    def canModify(self, REQUEST=None):
        """
        Checks if the current user has the permission to modify
        contacts
        """
        return getSecurityManager().checkPermission('Modify contact information', self)

    def UTF8ToLatin(self, string):
        return unicode(string, 'utf-8').encode('latin-1')

    def searchLDAPFilter(self, cn='', mail='', o='', REQUEST=None):
        """
        Convert latin to in the search parameters UTF and do the search
        itself.
        """
        params = { 'cn':cn, 'mail':mail, 'o':o }
        for attr in params.keys():
            if params[attr] == '':
                del params[attr]
            else:
                params[attr] = self._latinToUTF8(params[attr])

        print 'params=' + str(params)
        return self._searchLDAPFilter(params)

    def buildResultSequence(self, dict):
        """
        Builds a sequence of sequence that we can use directly with dtml-in
        based on the entry's attributes passed in a dict.
        """
        sequence = []
        for attr in result_attrs:
            if dict.has_key(attr[1]) and dict[attr[1]]:
                sequence.append([attr[0], dict[attr[1]]])
        return sequence


    ##
    # "Internal" functions (not used by anything else than the python
    # methods of this class):

    def _setupLDAPFilters(self):
        """ Construct the needed LDAP filter objects """
        # The filter used by the search form
        self._searchLDAPFilter = LDAPFilter('searchFilter', '', self.connection_id,
                                          'SUBTREE', self.basedn,
                                          'o cn mail',
                                          '(&<dtml-if mail>(mail=*<dtml-var mail>*)</dtml-if>\
                                           <dtml-if o>(o=*<dtml-var o>*)</dtml-if> \
                                           <dtml-if cn>(cn=*<dtml-var cn>*)</dtml-if> \
                                           (!(objectclass=organization)))');

        # The search filter used to fetch single entries
        self.getEntryLDAPFilter = LDAPFilter('entryFilter', '', self.connection_id,
                                             'SUBTREE', self.basedn,
                                             'clause', '<dtml-var clause>');

    def _getRDNFromDN(self, dn):
        """ Extracts the RDN from a DN """
        offset = dn.find(',')
        return offset > -1 and dn[:offset] or dn

    def _checkAttributes(self, dict):
        """
        Tries to construct a dictionary of attributes from the REQUEST
        object.

        Does some checks and modifications and returns a tuple of a message
        indicating the possible failure and the dictionary on success.
        """
        # cn is the only mandatory attribute.
        if not dict.has_key('cn') or not dict['cn']:
            return ('Define a name!', None)

        # Some modifications to the dict..
        dict['cn'] = self._removeTrailingWhitespace(str(dict['cn']))
        # We'll guess that the last word of the name is the surname
        names = str(dict['cn']).split()
        if(len(names) > 1):
            dict['sn'] = " ".join(names[-1:])
        if dict.has_key('postaladdress'):
            dict['postaladdress'] = \
              self._encodePostaladdress(postaladdress = dict['postaladdress'])

        entryAttrs = {}
        for attr in mod_attrs:
            if dict.has_key(attr) and dict[attr] != '':
                entryAttrs[attr] = self._latinToUTF8(dict[attr])

        return (None, entryAttrs)

    def _encodePostaladdress(self, postaladdress):
        """
        Changes newlines to $, used to encode the postaladdress before
        putting it into the database.
        """
        string = str(postaladdress)
        return string.replace('\n', '$')

    def _removeTrailingWhitespace(self, string):
        """ Removes tabs and spaces from the end of the string """
        while string[-1:] == ' ' or string[-1:] == '\t':
            string = string.rstrip()
        return string

    def _latinToUTF8(self, string):
        return unicode(string, 'latin-1').encode('utf-8')

